#property copyright "SK-FX tester"
#property link      "http://www.tradersforum.net.ru/forum/balash-m3616.html"

#property indicator_chart_window

#include <Tester.mqh>

#define HST_HEADER 148
#define HST_RECORD 44

extern int MaxBars = 500;

datetime ta[];
double oa[],la[],ha[],ca[],va[];
int handles[], pos[];
bool error = false;

int init()
{
   if (Period() != PERIOD_M1)
   {
      error = true;
      Alert("    M1");
   }
}

int deinit()
{
   for (int i = 0; i < periodsLength; i++)
      if (handles[i] != -1)
         FileClose(handles[i]);
}

int start()
{
   if (error)
      return (-1);
   static bool first = true;
   if (first)
   {
      first = false;
      ConvertMinutes();
      return (0);
   }

   int countedBars = IndicatorCounted();
   if (countedBars < 0)
      return (-1);
   int limit = Bars-1-countedBars;
   
   UpdateHistories(limit);   
}

void ConvertMinutes()
{
   periodsLength = ArraySize(periods);
   ArrayResize(ta,periodsLength);
   ArrayResize(oa,periodsLength);
   ArrayResize(la,periodsLength);
   ArrayResize(ha,periodsLength);
   ArrayResize(ca,periodsLength);
   ArrayResize(va,periodsLength);
   ArrayResize(handles,periodsLength);
   ArrayResize(pos,periodsLength);
   for (int i=0; i<periodsLength; i++)
      ta[i] = EMPTY_VALUE;
   
   int handle = FileOpenHistory(Symbol()+"1.hst",FILE_BIN|FILE_READ);
   if (handle == -1)
   {
      Alert("   ");
      return (-1);
   }
   FileSeek(handle,HST_HEADER,SEEK_SET);
   while (!FileIsEnding(handle))
   {
      datetime t = FileReadInteger(handle);
      double o = FileReadDouble(handle);
      double l = FileReadDouble(handle);
      double h = FileReadDouble(handle);
      double c = FileReadDouble(handle);
      double v = FileReadDouble(handle);
      if (t >= Time[0])
         break;
      MakeHistories(t,o,l,h,c,v);
   }
   FileClose(handle);
   RewriteMaxBarsHistories();
   WriteLastBars();
   UpdateHistories(0);
   Alert("  ");
}

void MakeHistories(datetime t, double o, double l, double h, double c, double v)
{
   for (int i = 0; i < periodsLength; i++)
      MakeHistoryFor(i,t,o,l,h,c,v);
}

void MakeHistoryFor(int index, datetime t, double o, double l, double h, double c, double v)
{
   int ps = periods[index]*60;
   datetime t0 = BarTime(index,t);

   if (ta[index] == EMPTY_VALUE || ta[index] != t0)
   {
      if (ta[index] == EMPTY_VALUE)
         handles[index] = FileOpen(Symbol()+periods[index]+".tmp",FILE_BIN|FILE_WRITE);
         //CreateHistoryFile(index,Symbol()+"t"+periods[index]+".hst",Symbol()+"t");
      else
         WriteHistoryRecord(index);   
      ta[index] = t0;
      oa[index] = o;
      la[index] = l;
      ha[index] = h;
      ca[index] = c;
      va[index] = v;
   }
   else
   {
      va[index] += v;
      if (l < la[index])
         la[index] = l;
      if (h > ha[index])
         ha[index] = h;
      ca[index] = c;
   }
}

void CreateHistoryFile(int index, string name, string symbol)
{
   int handle = FileOpenHistory(name,FILE_BIN|FILE_WRITE);
   handles[index] = handle;
   if (handle == -1)
   {
      Alert("    "+name);
      return;
   }
   int unused[13] = {0};
   FileWriteInteger(handle, 400);
   FileWriteString(handle, "(C)opyright 2003, MetaQuotes Software Corp.", 64);
   FileWriteString(handle, symbol, 12);
   FileWriteInteger(handle, periods[index]);
   FileWriteInteger(handle, Digits);
   FileWriteInteger(handle, 0);
   FileWriteInteger(handle, 0);
   FileWriteArray(handle, unused, 0, ArraySize(unused));
   pos[index] = FileTell(handle);
}

void WriteHistoryRecord(int index)
{
   int handle = handles[index];
   FileWriteInteger(handle, ta[index]);
   FileWriteDouble(handle, oa[index]);
   FileWriteDouble(handle, la[index]);
   FileWriteDouble(handle, ha[index]);
   FileWriteDouble(handle, ca[index]);
   FileWriteDouble(handle, va[index]);
}

void WriteLastBars()
{
   for (int i = 0; i < periodsLength; i++)
   {
      pos[i] = FileTell(handles[i]);
      WriteHistoryRecord(i);
      FileFlush(handles[i]);
   }
}

void RewriteMaxBarsHistories()
{
   for (int i = 0; i < periodsLength; i++)
   {
      RewriteMaxBarsHistory(i);
   }
}

void RewriteMaxBarsHistory(int index)
{
   FileClose(handles[index]);
   int handle = FileOpen(Symbol()+periods[index]+".tmp",FILE_BIN|FILE_READ);
   CreateHistoryFile(index, Symbol()+periods[index]+".hst",Symbol());
   int fBars = FileSize(handle)/HST_RECORD;
   if (fBars > MaxBars)
      fBars = MaxBars;
   FileSeek(handle,-fBars*HST_RECORD,SEEK_END);
   while (fBars > 0)
   {
      FileWriteInteger(handles[index],FileReadInteger(handle));
      FileWriteDouble(handles[index],FileReadDouble(handle));
      FileWriteDouble(handles[index],FileReadDouble(handle));
      FileWriteDouble(handles[index],FileReadDouble(handle));
      FileWriteDouble(handles[index],FileReadDouble(handle));
      FileWriteDouble(handles[index],FileReadDouble(handle));
      fBars--;
   }
   FileClose(handle);
   FileDelete(Symbol()+periods[index]+".tmp");
}

void UpdateHistories(int limit)
{
   for (int i = 0; i < periodsLength; i++)
      UpdateHistoryFor(i,limit);
}

void UpdateHistoryFor(int index, int limit)
{
   int ps = periods[index]*60;
   FileSeek(handles[index],pos[index],SEEK_SET);
   
   for (int bar = limit; bar >= 0; bar--)
   {
      datetime t0 = BarTime(index, Time[bar]);
      
      if (ta[index] == t0)
      {
         va[index] += Volume[bar];
         if (Low[bar] < la[index])
            la[index] = Low[bar];
         if (High[bar] > ha[index])
            ha[index] = High[bar];
         ca[index] = Close[bar];
      }
      else
      {
         WriteHistoryRecord(index);
         ta[index] = t0;
         oa[index] = Open[bar];
         la[index] = Low[bar];
         ha[index] = High[bar];
         ca[index] = Close[bar];
         va[index] = Volume[bar];
      }
   }
   pos[index] = FileTell(handles[index]);
   WriteHistoryRecord(index);
   va[index] -=  Volume[0];
   
   FileFlush(handles[index]);
}

datetime BarTime(int index, datetime t)
{
   int ps = periods[index]*60;
   datetime t0;
   datetime today = StrToTime(StringSubstr(TimeToStr(t),0,10));
   switch(periods[index])
   {
   case PERIOD_D1:
      t0 = today;
      break;
   case PERIOD_W1:
      t0 = today;
      while(TimeDayOfWeek(t0) != 0)
         t0 -= PERIOD_D1*60;
      break;
   case PERIOD_MN1:
      t0 = StrToTime(StringSubstr(TimeToStr(t),0,8)+"01");
      break;
   default:
      int temp = t/ps;
      t0 = temp*ps;
   }
   return (t0);
}

